{*************************************************************************}
{*             Bio rhythms viewer   BIOMAIN.PAS used by BIO.DPR          *}
{*            -------------------                                        *}
{*          by Michael Kochiashvili                                      *}
{*                 1995                                                  *}
{* email: Kochini@iberiapac.ge                                           *}
{*************************************************************************}

unit Biomain;

interface

uses
  SysUtils, WinTypes, WinProcs, Messages, Classes, Graphics, Controls,
  Forms, Dialogs, TpDate, ExtCtrls, Tabs, StdCtrls, Grids,
  Calendar, Spin;

{ --------- Bio rhythms types and constants -------------------------}
type
  RhythmType = ( NoRhythm, Physical, Emotional, Intelectual);
  RhythmPeriodType = array[ NoRhythm..Intelectual] of byte;
  RhythmColorType  = array[NoRhythm..Intelectual] of TColor;
  RhythmStatusType = (
    NoStatus,
    Positive,    { when rhythm is positive}
    Negative,    { ............. negative}
    PosToNeg,    { when rhythm changes from positive to negative}
    NegToPos     { ....................... negative to positive}
    );
  RhythmStatusArrayType = array[ NoRhythm..Intelectual] of RhythmStatusType;

const
  RhythmPeriod : RhythmPeriodType =( 0, 23, 28, 33); { rhythms period in days}
  RhythmColor : RhythmColorType = ( clWhite, clRed, clBlue, clGreen);
  RithmStatusScreenChar : array[ NoStatus..NegToPos] of char =
    { NoStatus  Positive  Negative  PosToNeg  NegToPos}
    ( ' ',      '+',      '-',      '',      ''     );
  RhythmStatusPrinterChar : array[ NoStatus..NegToPos] of char =
    { NoStatus  Positive  Negative  PosToNeg  NegToPos}
	( ' ',      '+',      '-',      'X',      'X'     );
{-------------------------------------------------------------------}

{ Zodiacal signs}
type
  ZodDataType = record
    FrD, FrM : integer; { ranges: from Date,Month ->}
    {ToD, ToM : integer;}      {    to Date,Month ( it is not necesary)}
    ZN : string[20]; { zodiac name}
    ZR  : PChar; { resource name}
    end;

  ZodArrayType = array[1..12] of ZodDataType;
const
  ZodArray : ZodArrayType = (
    ( FrD:23; FrM:01; {ToD:22; ToM:02;} ZN:'Aquarius'; ZR:'AQUARIUS'),
    ( FrD:23; FrM:02; {ToD:22; ToM:03;} ZN:'Pisces';   ZR:'PISCES'),
    ( FrD:23; FrM:03; {ToD:22; ToM:04;} ZN:'Aries';    ZR:'ARIES'),
    ( FrD:23; FrM:04; {ToD:22; ToM:05;} ZN:'Taurus';   ZR:'TAURUS'),
    ( FrD:23; FrM:05; {ToD:22; ToM:06;} ZN:'Gemini';   ZR:'GEMINI'),
    ( FrD:23; FrM:06; {ToD:22; ToM:07;} ZN:'Cancer';   ZR:'CANCER'),
    ( FrD:23; FrM:07; {ToD:22; ToM:08;} ZN:'Leo';      ZR:'LEO'),
    ( FrD:23; FrM:08; {ToD:22; ToM:09;} ZN:'Virgo';    ZR:'VIRGO'),
    ( FrD:23; FrM:09; {ToD:22; ToM:10;} ZN:'Libra';    ZR:'LIBRA'),
    ( FrD:23; FrM:10; {ToD:22; ToM:11;} ZN:'Scorpio';  ZR:'SCORPIO'),
    ( FrD:23; FrM:11; {ToD:22; ToM:12;} ZN:'Sagitari'; ZR:'SAGITARI'),
    ( FrD:23; FrM:12; {ToD:22; ToM:01;} ZN:'Carpicor'; ZR:'CARPICOR')
    );
{-------------------------------------------------------------------}

type
  TMainF = class(TForm)
    TabSet1: TTabSet;
    Bevel1: TBevel;
    Notebook1: TNotebook;

    { Name }
    Label1: TLabel;
    Edit1: TEdit;
    Bevel2: TBevel;

    { Group "Births date" }
    GBBirthDate: TGroupBox;
    BirthDayCal: TCalendar;  { Calendar}
    BBDMonth: TBevel;        { Month controls}
    LBDMonth: TLabel;
    SBBDMonth: TSpinButton;
    BBDYear: TBevel;         { Year controls}
    LBDYear: TLabel;
    SBBDYear: TSpinButton;

    { Bio Graph page}
    BiorGraph: TImage; { Bio rhythms graph image}
    LPhys  : TLabel;   { Rhythm name labels}
    LEmot  : TLabel;
    LIntel : TLabel;
    SBMaxDays: TSpinButton;
    Bevel3: TBevel;
    LMaxDay: TLabel;
    LMD: TLabel;


    RightPanel: TPanel;

    { Group "View from date"}
    GBViewFrom: TGroupBox;
    ViewFromCal: TCalendar; { View from date calendar}
    BVFDay: TBevel;         { Day controls}
    LVFDay: TLabel;
    SBVFDay: TSpinButton;
    BVFMonth: TBevel;       { Month controls}
    LVFMonth: TLabel;
    SBVFMonth: TSpinButton;
    BVFYear: TBevel;        { Year controls}
    LVFYear: TLabel;
    SBVFYear: TSpinButton;
    BSetCurrentD: TButton;  { Set calendar to current date}
    LVFTit1: TLabel;
    LVFTit2: TLabel;


    { Bio calendar page}
    DGBioCal: TDrawGrid;   { Calendar}
    CCritDays: TCheckBox;
    CRhytSign: TCheckBox;
    CSignCh: TCheckBox;
    BSigns: TBevel;      { Bio signs}
    ImSigns: TImage;
    BSignCh: TBevel;     { Bio sign change}
    ImSignCh: TImage;
    LPhys1 : TLabel;   { Rhythm name labels}
    LEmot1 : TLabel;
    LIntel1: TLabel;

    { Critical day group}
    Bevel9: TBevel;
    CriticalDayCal: TCalendar;  { critical day calendar}
    LCrDay: TLabel;             { Day, month, year title}
    LCrMonth: TLabel;
    LCrYear: TLabel;
    BSetViewFrom: TButton;
    BResetToVF: TButton;

    { Group "Search for"}
    GBSearchF: TGroupBox;
    RBSingle: TRadioButton;
    RBDouble: TRadioButton;
    RBTriple: TRadioButton;

    { Search critical day buttons}
    BSearchNext: TButton;
    BSearcPrev: TButton;

    { Group "Include rhythm"}
    GBIncludeR: TGroupBox;
    CBPhys: TCheckBox;
    CBEmot: TCheckBox;
    CBIntel: TCheckBox;

    { Finded rhythm status bevel}
    Bevel10: TBevel;
    Bevel11: TBevel;
    ImPhysSt: TImage;  { Bio status images}
    ImEmotSt: TImage;
    ImInteSt: TImage;
    LPhys2: TLabel;    { Bio rhythm names}
    LEmot2: TLabel;
    LIntel2: TLabel;


    { Statistics labels etc}
    ImZodiac: TImage;
    BZodiac: TBevel;
    ZodName: TLabel;
    ZodTitle: TLabel;
    Label14: TLabel;
    Label15: TLabel;
    GroupBox5: TGroupBox;
    Label16: TLabel;
    Label17: TLabel;
    Label18: TLabel;
    Label19: TLabel;
    Label20: TLabel;
    Label21: TLabel;
    Label22: TLabel;
    Label23: TLabel;
    Label24: TLabel;
    Label25: TLabel;
    Label26: TLabel;

    { About page}
    AboutPanel: TPanel;
    Image7: TImage;
    Label29: TLabel;
    Label30: TLabel;
    Label31: TLabel;
    Bevel4: TBevel;
    Bevel5: TBevel;
    Bevel6: TBevel;
    Label2: TLabel;

    procedure FormCreate(Sender: TObject);
    procedure TabSet1Click(Sender: TObject);
    procedure SBBDMonthDownClick(Sender: TObject);
    procedure SBBDMonthUpClick(Sender: TObject);
    procedure SBBDYearUpClick(Sender: TObject);
    procedure SBBDYearDownClick(Sender: TObject);
    procedure BirthDayCalChange(Sender: TObject);
    procedure FormActivate(Sender: TObject);
    procedure SBVFMonthDownClick(Sender: TObject);
    procedure SBVFMonthUpClick(Sender: TObject);
    procedure SBVFYearDownClick(Sender: TObject);
    procedure SBVFYearUpClick(Sender: TObject);
    procedure ViewFromCalChange(Sender: TObject);
    procedure Notebook1PageChanged(Sender: TObject);
    procedure DGBioCalDrawCell(Sender: TObject; Col, Row: Longint;
      Rect: TRect; State: TGridDrawState);
    procedure CCritDaysClick(Sender: TObject);
    procedure CRhytSignClick(Sender: TObject);
    procedure CSignChClick(Sender: TObject);
    procedure ImSignsClick(Sender: TObject);
    procedure ImSignChClick(Sender: TObject);
    procedure CriticalDayCalChange(Sender: TObject);
    procedure BSetViewFromClick(Sender: TObject);
    procedure BResetToVFClick(Sender: TObject);
    procedure BSetCurrentDClick(Sender: TObject);
    procedure BSearcPrevClick(Sender: TObject);
    procedure BSearchNextClick(Sender: TObject);
    procedure SBVFDayDownClick(Sender: TObject);
    procedure SBVFDayUpClick(Sender: TObject);
    procedure SBMaxDaysDownClick(Sender: TObject);
    procedure SBMaxDaysUpClick(Sender: TObject);
  private
    { Private declarations }
    DaysInGraph : integer; { days in bio grah}
    TmpCan : TCanvas;
    BirthDate, CurDate, FromDate : TDate;
    BmpRPlus, BmpRMinus, BmpRUp, BmpRDown, TmpBmp : TBitMap; { +,-, Up, Down: pictures}
    BmpRCreate : boolean; { true when BmpR are created}
    { Search parameters}
    SearchDate : TDate;
    SearchForward : boolean;

    ZodiacBMP : TBitMap; { zodiac picture}

    procedure GetRhythmStatus( BDate, FDate : TDate; { birth and current dates}
                              CRithm : RhythmType;  { rhythm we interesting}
                          var RStatus : RhythmStatusType { rhythm status} );
    {------------------- Bio Graph}
    procedure DrawBRGraphic; { draw Bio Graphic}
    {------------------- Bio Calendar}
    procedure DrawBRCalendar;
    {------------------- Search critical days}
    { Resets searching settings}
    procedure ResetSearchProcess;
    { Begin search critical day using settings "Search for", "Include rhythm"
      from date in CriticalDayCal}
    procedure SearchCriticalDay;

    { Draw About page}
    procedure DrawAbout;

  public
    { Public declarations }
  end;

var
  MainF: TMainF;

implementation

{$R *.DFM}

{------------- BioRhythm calculations ----------------------------------}
{ }
procedure TMainF.GetRhythmStatus(
            BDate, FDate : TDate; { birth and current dates}
            CRithm : RhythmType;  { rhythm we interesting}
            var RStatus : RhythmStatusType { rhythm status}         );
var
  DayDif, DayMod, HalfRhythm : TDate;
begin
  DayDif := FDate - BDate;
  DayMod := DayDif MOD RhythmPeriod[ CRithm];
  HalfRhythm := RhythmPeriod[ CRithm] DIV 2;
  if DayMod = 0 then               RStatus := NegToPos { New cycle starts}
  else if DayMod < HalfRhythm then RStatus := Positive
  else if DayMod = HalfRhythm then RStatus := PosToNeg
  else if DayMod > HalfRhythm then RStatus := Negative;
end; { GetRhythmStatus}

{----------------------------------------------------------------------}


{ Draw Bio Graph}
procedure TMainF.DrawBRGraphic;
var
  ImH, ImW, ImMidY : integer;
  Cou, CouPos, PixPerDay, DayMid : integer;
  DaysInPix, SinDat, XScale : real;
  ColorSw : boolean;
  DayCou : TDate;
  RCou : RhythmType;
  RStatus : RhythmStatusType;
  OldCursor : TCursor;

  { If day is first day of month returns Day num and month name,
    else day num each fifth day
  }
  function DayStr( DT : TDate) : string;
  var D, M, Y : integer; S : string[5];
  begin
    S := '';
    DateToDMY( DT, D, M, Y);
    if D = 1 then S := ShortMonthNames[ M]
    else begin
      if D MOD 5 = 0 then begin
        S := Long2Str( D);
        end;
      end;
    DayStr := S;
  end; { DayStr }

  { Draws specified rhythm}
  procedure DrawCycle( CRithm : RhythmType);
  var
    Cou, YVal : integer;
    DayCou : TDate;
  begin
    Cou := 0;
    DayCou := FromDate - BirthDate;

    SinDat := 2 * Pi / RhythmPeriod[ CRithm];

    with BioRGraph.Canvas do begin
      Pen.Width := 3;
      Pen.Color := RhythmColor[ CRithm];
      CouPos := 0;
      YVal := ImMidY - trunc( XScale * sin( SinDat * DayCou) * ImMidY);
      MoveTo( Cou, YVal);
      while Cou <= ImW + PixPerDay do begin
        if DayCou >= 0 then begin
          YVal := ImMidY - trunc( XScale * sin( SinDat * DayCou) * ImMidY);
          if DayCou = 0 then MoveTo( Cou, YVal)
          else               LineTo( Cou, YVal);
          end;
        inc( DayCou);
        inc( Cou, PixPerDay);
        end;
      end;
  end; { DrawCycle}

  { Put text without background change}
  procedure CopyText( X, Y : integer; S : string; Can : TCanvas);
  var BM : TBitMap; Rs, Rd : TRect;
  begin
    BM := TBitMap.Create;
    BM.Canvas.Font := Can.Font; { Copy Dest. canvas font settings}
    BM.Height := BM.Canvas.TextHeight( S); BM.Width := BM.Canvas.TextWidth( S);
    BM.Canvas.TextOut( 0, 0, S);
    with Rs do begin
      Left := X; Top := Y; Right := X+BM.Width; Bottom := Y+BM.Height;
      end;
    with Rd do begin
      Left := 0; Top := 0; Right := BM.Width; Bottom := BM.Height;
      end;
    BM.Canvas.CopyMode := cmSrcAnd;
    BM.Canvas.CopyRect( Rd, BioRGraph.Canvas, Rs);
    BioRGraph.Canvas.Draw( X, Y, BM);
    BM.Free;
  end; { CopyText}

begin
  { Set wait cursor}
  OldCursor := Cursor;
  Screen.Cursor := crHourGlass;

  ImH := BioRGraph.Height;
  ImW := BioRGraph.Width;
  ImMidY := BioRGraph.Height DIV 2;

  PixPerDay := ImW DIV DaysInGraph;   { pixels in 1 day}
  DayMid := PixPerDay DIV 2;
  XScale := 0.85;


  { Drawing grid}
  ColorSw := FromDate MOD 2 = 0;
  with BioRGraph.Canvas do begin
    Pen.Width := PixPerDay;
    CouPos := DayMid;
    while CouPos <= ImW + PixPerDay do begin
      if ColorSw then Pen.Color := clAqua
      else            Pen.Color := clWhite;
      MoveTo( CouPos, 0); LineTo( CouPos, ImH);
      inc( CouPos, PixPerDay);
      ColorSw := NOT ColorSw;
      end;
    end;

  { Writing days & critical nums}
  with BioRGraph.Canvas do begin
    Font.Name := 'MS Sans Serif'{'Small Fonts'};
    Font.Color := clBlack;
    Font.Size := 8;
    DayCou := FromDate;
    CouPos := 0;
    while CouPos <= ImW do begin
      CopyText( CouPos,
                ImH - TextHeight( 'M')- 2,
                DayStr( DayCou),
                BioRGraph.Canvas);
      Cou := 0;
      for RCou := Physical to Intelectual do begin
        GetRhythmStatus( BirthDate, DayCou, RCou, RStatus);
         if RStatus IN [ PosToNeg, NegToPos] then inc( Cou);
         end;
      if Cou <> 0 then CopyText( CouPos, 2, Long2Str( Cou), BioRGraph.Canvas);
      inc( CouPos, PixPerDay);
      inc( DayCou);
      end;
    end;

  { Drawing X-axis}
  with BioRGraph.Canvas do begin
    Brush.Color := clWhite;
    Pen.Width := 2;
    Pen.Color := clBlack;
    MoveTo( 0, ImMidY); LineTo( ImW, ImMidY);
    end; { BioRGraph.Canvas}

  { Draw grid lines on X-axis}
  with BioRGraph.Canvas do begin
    Pen.Width := 1;
    Pen.Color := clTeal;
    CouPos := 0;
    while CouPos <= ImW do begin
      MoveTo( CouPos, ImMidY - 4); LineTo( CouPos, ImMidY + 4);
      inc( CouPos, PixPerDay);
      end;
    end;


  { Draw rhythms}
  for RCou := Physical to Intelectual do begin
    DrawCycle( RCou);
    end;

  { Restore normal cursor}
  Screen.Cursor := OldCursor;
end; { TMainF.DrawBRGraphic}


{ Draw bio rhythms calendar}
procedure TMainF.DrawBRCalendar;
var
  OldCursor : TCursor;
begin
  { Set wait cursor}
  OldCursor := Cursor;
  Screen.Cursor := crHourGlass;

  DGBioCal.Repaint;

  { Restore normal cursor}
  Screen.Cursor := OldCursor;
end; { TMainF.DrawBRCalendar}


procedure TMainF.FormCreate(Sender: TObject);
begin
  TabSet1.Tabs := Notebook1.Pages;

  { Create zodiac bitmap}
  ZodiacBMP := TBitMap.Create;
  ZodiacBMP.Width := 120;   ZodiacBMP.Height := 120;

  { Set Birth dates and Calendar1 to my birth date}
  with BirthDayCal do begin
    Label17.Caption := FormatDateTime( 'dd-mm-yy', CalendarDate);
    CurDate := DMYToDate( TDate( Day), TDate( Month), TDate( Year));
    Day := 8; Month := 1; Year := 1960;
    BirthDate := DMYToDate( TDate( Day), TDate( Month), TDate( Year));

    UpdateCalendar;
    end;

  { Set View from dates}
  with ViewFromCal do begin
    FromDate := DMYToDate( TDate( Day), TDate( Month), TDate( Year));
    UpdateCalendar;
    end;

  { Critical days calendar}
  with CriticalDayCal do begin
    LCrMonth.Caption := LongMonthNames[ Month];
    LCrYear.Caption := Long2Str( Year);
    LCrDay.Caption := Long2Str( Day);
    end;

  { Set rhythm labels colors}
  LPhys.Font.Color     := RhythmColor[ Physical];
  LEmot.Font.Color     := RhythmColor[ Emotional];
  LIntel.Font.Color    := RhythmColor[ Intelectual];
  LPhys1.Font.Color    := RhythmColor[ Physical];
  LEmot1.Font.Color    := RhythmColor[ Emotional];
  LIntel1.Font.Color   := RhythmColor[ Intelectual];
  LPhys2.Font.Color    := RhythmColor[ Physical];
  LEmot2.Font.Color    := RhythmColor[ Emotional];
  LIntel2.Font.Color   := RhythmColor[ Intelectual];
  CBPhys.Font.Color    := RhythmColor[ Physical];
  CBEmot.Font.Color    := RhythmColor[ Emotional];
  CBIntel.Font.Color   := RhythmColor[ Intelectual];

  BmpRCreate := false; { BmpR... are not created}

  DaysInGraph := 31; { set bio graph days}
  LMaxDay.Caption := Long2Str( DaysInGraph);

  SearchDate := FromDate; { Search parameters}

  { Load icon for About page and center labels}
  Image7.Picture.Icon.Handle := LoadIcon( HInstance, 'MAINICON');

  { Center labels for other displays}
  Image7.Left  := ( AboutPanel.Width - Image7.Width) DIV 2;
  Label29.Left := ( AboutPanel.Width - Label29.Width) DIV 2;
  Label30.Left := ( AboutPanel.Width - Label30.Width) DIV 2;
  Label31.Left := ( AboutPanel.Width - Label31.Width) DIV 2;

end; { TMainF.FormCreate}

{ Change notebook page when click tabset}
procedure TMainF.TabSet1Click(Sender: TObject);
begin
  Notebook1.PageIndex := TabSet1.TabIndex;
end; { TMainF.TabSet1Click}

procedure TMainF.SBBDMonthDownClick(Sender: TObject);
begin
  if BirthDayCal.Month > 1 then BirthDayCal.PrevMonth;
end; { TMainF.SpinButton2DownClick}

procedure TMainF.SBBDMonthUpClick(Sender: TObject);
begin
  if BirthDayCal.Month < 12 then BirthDayCal.NextMonth;
end; { TMainF.SpinButton2UpClick}

procedure TMainF.SBBDYearUpClick(Sender: TObject);
begin
  BirthDayCal.NextYear;
end; { TMainF.SpinButton1UpClick}

procedure TMainF.SBBDYearDownClick(Sender: TObject);
begin
  BirthDayCal.PrevYear;
end; { TMainF.SpinButton1DownClick}

{ When change birth day calendar}
procedure TMainF.BirthDayCalChange(Sender: TObject);
var
  DateDif : TDate;
  D, M, Y : integer;
  TmpBmp : TBitMap;
begin
  with BirthDayCal do begin
    LBDMonth.Caption := LongMonthNames[ Month];
    LBDYear.Caption := Long2Str( Year);
    BirthDate := DMYToDate( TDate( Day), TDate( Month), TDate(Year));
    Label15.Caption := FormatDateTime( 'dddddd', CalendarDate);
    { Calc years and days}
    DateDif := CurDate - BirthDate;
    Label21.Caption := Long2Str( DateDif);
    DateDiff( CurDate, BirthDate, D, M, Y);
    Label20.Caption := Long2Str( Y);
    end;

  { Update statistics}
  ViewFromCalChange(Sender);
end; { TMainF.BirthDayCalChange}

procedure TMainF.FormActivate(Sender: TObject);
begin
  {$IFDEF DEBUG} { When DEBUGING}
  TabSet1.TabIndex := 4;
  {$ELSE}
  TabSet1.TabIndex := 0;
  {$ENDIF}
  Notebook1.PageIndex := TabSet1.TabIndex;
end; { TMainF.FormActivate}

procedure TMainF.SBVFMonthDownClick(Sender: TObject);
begin
  if ViewFromCal.Month > 1 then ViewFromCal.PrevMonth;
end; { TMainF.SpinButton3DownClick}

procedure TMainF.SBVFMonthUpClick(Sender: TObject);
begin
    if ViewFromCal.Month < 12 then ViewFromCal.NextMonth;
end; { TMainF.SpinButton3UpClick}

procedure TMainF.SBVFYearDownClick(Sender: TObject);
begin
  ViewFromCal.PrevYear;
end; { TMainF.SpinButton4DownClick}

procedure TMainF.SBVFYearUpClick(Sender: TObject);
begin
  ViewFromCal.NextYear;
end; { TMainF.SpinButton4UpClick}

{ When View From calendar changed}
procedure TMainF.ViewFromCalChange(Sender: TObject);
var
  DateDif : TDate;
  D, M, Y : integer;
begin
  with ViewFromCal do begin
    LVFDay.Caption := Long2Str( Day);
    LVFMonth.Caption := ShortMonthNames[ Month];
    LVFYear.Caption := Long2Str( Year);
    Label19.Caption := FormatDateTime( 'dd-mm-yy', CalendarDate);
    FromDate := DMYToDate( TDate( Day), TDate( Month), TDate( Year));
    { Calc years and days}
    DateDif := FromDate - BirthDate;
    Label23.Caption := Long2Str( DateDif);
    DateDiff( FromDate, BirthDate, D, M, Y);
    Label22.Caption := Long2Str( Y);
    end;
  { Redraw current page}
  case Notebook1.PageIndex of
    1: begin { rhythms graphic}
      DrawBRGraphic;
      end;
    2: begin { rhythms calendar}
      DrawBRCalendar;
      end;
    end;
end; { TMainF.ViewFromCalChange}

{ When notebook page changed}
procedure TMainF.Notebook1PageChanged(Sender: TObject);

  procedure BmpRhythmsCreate;
  begin
    if NOT BmpRCreate then begin
      BmpRCreate := true;
      BmpRPlus := TBitMap.Create; BmpRPlus.Handle := LoadBitMap( HInstance, 'RPLUS');
      BmpRMinus := TBitMap.Create; BmpRMinus.Handle := LoadBitMap( HInstance, 'RMINUS');
      BmpRUp := TBitMap.Create; BmpRUp.Handle := LoadBitMap( HInstance, 'RUP');
      BmpRDown := TBitMap.Create; BmpRDown.Handle := LoadBitMap( HInstance, 'RDOWN');
      TmpBmp := TBitMap.Create; TmpBmp.Width := BmpRUp.Width; TmpBmp.Height := BmpRUp.Height;
      end;
  end; { BmpRhythmsCreate}

  procedure LoadZodiac;
  var
    TmpZBmp : TBitMap;
    TmpR : TRect;
    aTop, aLeft : integer;
    ZInd : integer;
    ZRName : PChar;

  begin
    with BirthDayCal do begin
      if Day >= ZodArray[ Month].FrD then ZInd := Month
      else                                ZInd := pred( Month);
      if ZInd = 0 then ZInd := 12;
      end;
    ZRName := ZodArray[ZInd].ZR;
    ZodName.Caption := ZodArray[ZInd].ZN;


    TmpZBmp := TBitMap.Create;
    TmpZBmp.Handle := LoadBitMap( HInstance, ZRName);

    aTop  := ( ZodiacBMP.Height - TmpZBmp.Height) DIV 2;
    aLeft := ( ZodiacBMP.Width  - TmpZBmp.Width)  DIV 2;
    TmpR := Rect( aLeft, aTop, aLeft + TmpZBmp.Width, aTop + TmpZBmp.Height);

    with ZodiacBMP.Canvas do begin
      Brush.Color := clSilver;
      FillRect( ClipRect);
      CopyMode := cmSrcAnd;
      CopyRect( TmpR, TmpZBmp.Canvas, TmpZBmp.Canvas.ClipRect);
      end;

    ImZodiac.Picture.Graphic := ZodiacBMP;

    TmpZBmp.Free;
  end; { LoadZodiac}

begin
  { Free BmpR... if not use}
  if (( Notebook1.PageIndex <> 2) OR ( Notebook1.PageIndex <> 3) )
        AND ( BmpRCreate) then begin
    { destroy BmpR...}
    BmpRPlus.Free; BmpRMinus.Free; BmpRUp.Free; BmpRDown.Free; TmpBmp.Free;
    BmpRCreate := false;
  end;

  case Notebook1.PageIndex of { draw bior graphic}
    1 : begin { Bio Graphic}
      DrawBRGraphic;
      end;
    2 : begin { Bio calendar}
      BmpRhythmsCreate;
      with ImSigns.Canvas do begin
        Brush.Color := clSilver;
        FillRect( ClipRect);
        Draw( 0, 0, BmpRPlus); Draw( 16, 0, BmpRMinus);
        end;
      with ImSignCh.Canvas do begin
        Brush.Color := clSilver;
        FillRect( ClipRect);
        Draw( 0, 0, BmpRUp); Draw( 16, 0, BmpRDown);
        end;
      end; {2}
    3 : begin { Critical days}
      BmpRhythmsCreate;
      end; { 3}
    4 : begin { statistics}
      LoadZodiac;
      end; { 4}
    5 : begin { About page}
      DrawAbout;
      end;
    end; { case}
end; { TMainF.Notebook1PageChanged}

{ Draw Bio status signs in calendar cell
 In CrCou returns number of critical days}
procedure DrawBioCalendarSigns( Col, Row: Longint; Rect: TRect; var CrCou : integer);
var
  SDate : string;
  CDate : TDate;
  IDate, IC : integer;
  RStatus : RhythmStatusType;
  RColor : TColor;
  RCou : RhythmType;
  SignStartX, SignStartY : integer;
begin
  CrCou := 0;
  SDate := MainF.ViewFromCal.CellText [ Col, Row];
  { Draw rhythm status}
  val( SDate, IDate, IC);
  CDate := DMYToDate( 1, MainF.ViewFromCal.Month, MainF.ViewFromCal.Year) + IDate - 1;
  SignStartX := Rect.Right - 3 * MainF.TmpBmp.Width;
  SignStartY := Rect.Top + ( Rect.Bottom - Rect.Top - MainF.TmpBmp.Height) div 2;
  for RCou := Physical to Intelectual do begin
    MainF.GetRhythmStatus( MainF.BirthDate, CDate, RCou, RStatus);
    if RStatus IN [PosToNeg, NegToPos] then inc( CrCou);
    with MainF.TmpBmp.Canvas do begin
      Brush.Color := RhythmColor[ RCou];
      FillRect( ClipRect);
      CopyMode := cmSrcPaint;
      case RStatus of
        Positive : CopyRect( ClipRect, MainF.BMPRPlus.Canvas, ClipRect);
        Negative : CopyRect( ClipRect, MainF.BMPRMinus.Canvas, ClipRect);
        PosToNeg : CopyRect( ClipRect, MainF.BMPRDown.Canvas, ClipRect);
        NegToPos : CopyRect( ClipRect, MainF.BMPRUp.Canvas, ClipRect);
        end; { case RStatus}
      if ( MainF.CRhytSign.Checked AND
         ( RStatus IN [ Positive, Negative]))  OR
         ( MainF.CSignCh.Checked AND
         ( RStatus IN [ PosToNeg, NegToPos]))  then
        MainF.DGBioCal.Canvas.Draw( SignStartX, SignStartY, MainF.TmpBmp);
      inc( SignStartX, MainF.TmpBmp.Width);
      end; { with MainF.TmpBmp.Canvas}
    end; { for RCou }
end; { DrawBioCalendarSigns}

{ Draw cell in bio calendar}
procedure TMainF.DGBioCalDrawCell(Sender: TObject; Col, Row: Longint;
  Rect: TRect; State: TGridDrawState);
var
  SDate : string;
  TOfs, CritCou : integer;
  OldPenMode : TPenMode;
begin
  SDate := ViewFromCal.CellText [ Col, Row];
  with Rect, DGBioCal.Canvas do
    if Row = 0 then begin
      TOfs := (Right - Left - TextWidth(SDate)) div 2;
      Brush.Color := DGBioCal.FixedColor;
      end
    else begin
      Brush.Color := DGBioCal.Color;
      TOfs := 2;
      end;

  with Rect, DGBioCal.Canvas do
    TextRect(Rect, Left + TOfs, Top + (Bottom - Top - TextHeight(SDate)) div 2, SDate);

  { Draw only no empty calendar cells}
  if ( Row <> 0) AND ( SDate <> '') then begin { calendar date cells}
    DrawBioCalendarSigns( Col, Row, Rect, CritCou);
    if ( CCritDays.Checked) AND ( CritCou <> 0) then begin
      with DGBioCal.Canvas do begin
        OldPenMode := Pen.Mode; Pen.Mode := pmXor;
        Pen.Color := clGray;
        Pen.Width := 3 * CritCou;
        with Rect do begin
          MoveTo( Left, Top); LineTo( Right, Bottom);
          MoveTo( Right, Top); LineTo( Left, Bottom);
          end;
        Pen.Mode := OldPenMode;
        end;
      end;
    end; { if ( Row <> 0) AND ( SDate <> '')}
{
  if ( Col = DrawGrid1.Col) AND ( Row = DrawGrid1.Row) then
    DrawGrid1.Canvas.DrawFocusRect( Rect);
}
end; { TMainF.DrawGrid1DrawCell}

procedure TMainF.CCritDaysClick(Sender: TObject);
begin
  DrawBRCalendar;
end; { TMainF.CheckBox1Click}

procedure TMainF.CRhytSignClick(Sender: TObject);
begin
  DrawBRCalendar;
end; { TMainF.CheckBox2Click}

procedure TMainF.CSignChClick(Sender: TObject);
begin
  DrawBRCalendar;
end; { TMainF.CheckBox3Click}

procedure TMainF.ImSignsClick(Sender: TObject);
begin
  CRhytSign.Checked := NOT CRhytSign.Checked;
end; { TMainF.Image2Click}

procedure TMainF.ImSignChClick(Sender: TObject);
begin
  CSignCh.Checked := NOT CSignCh.Checked;
end; { TMainF.Image3Click}

{ When search calendar changed}
procedure TMainF.CriticalDayCalChange(Sender: TObject);
begin
  with CriticalDayCal do begin
    LCrDay.Caption := Long2Str( Day);
    LCrMonth.Caption := LongMonthNames[ Month];
    LCrYear.Caption := Long2Str( Year);
    end;
  ResetSearchProcess;
end; { TMainF.Calendar3Change}

{ Set view from date button}
procedure TMainF.BSetViewFromClick(Sender: TObject);
begin
  ViewFromCal.Day   := CriticalDayCal.Day;
  ViewFromCal.Month := CriticalDayCal.Month;
  ViewFromCal.Year  := CriticalDayCal.Year;
end; { TMainF.Button1Click}

{ Reset to view from date}
procedure TMainF.BResetToVFClick(Sender: TObject);
begin
  CriticalDayCal.Day   := ViewFromCal.Day;
  CriticalDayCal.Month := ViewFromCal.Month;
  CriticalDayCal.Year  := ViewFromCal.Year;
  ResetSearchProcess;
end; { TMainF.Button2Click}

{ Set view from date calendar to current date}
procedure TMainF.BSetCurrentDClick(Sender: TObject);
var D, M, Y : integer;
begin
  DateToDMY( CurDate, D, M, Y);
  with ViewFromCal do begin
    Day := D; Month := M; Year := Y;
    end;
end; { TMainF.Button3Click}


{ Resets searching settings}
procedure TMainF.ResetSearchProcess;
begin
  with CriticalDayCal do
    SearchDate := DMYToDate( Day, Month, Year);
end; { TMainF.ResetSearchProcess}



{ Begin search critical day using settings "Search for", "Include rhythm"
  from date in CriticalDayCal}
procedure TMainF.SearchCriticalDay;
type TCritCou = ( None, Single, Double, Triple);
var
  D, M, Y : integer;
  TmpSrch : TDate;
  Find : boolean;
  RythmSet, CriticalSet : set of RhythmType;
  RStatus : RhythmStatusType;
  CritC : TCritCou;
  RCou : RhythmType;
  RStat : RhythmStatusType;
  OldCursor : TCursor;

  function FindCritical : boolean;
  var
    CCou : integer;
    RCou : RhythmType;
    RS : RhythmStatusType;
    TmpFind : boolean;
  begin
    FindCritical := false;
    CriticalSet := [];
    CCou := 0;
    for RCou := Physical to Intelectual do begin
      GetRhythmStatus( BirthDate, TmpSrch, RCou, RS);
      if RS IN [PosToNeg, NegToPos] then begin
        CriticalSet := CriticalSet + [ RCou];
        inc( CCou);
        end; { if RS IN }
      end; { for RCou }

    { Exit if no critical days}
    if CCou < ord( CritC) then exit;

    case CritC of
      Single : FindCritical := ( CriticalSet * RythmSet <= RythmSet) AND
                               ( CriticalSet * RythmSet <> []);
      Double : FindCritical := (CCou = 2) AND
                               (( CriticalSet * RythmSet = RythmSet) OR
                                ( CriticalSet * RythmSet = CriticalSet))
                               AND ( CriticalSet * RythmSet <> []);
      Triple : FindCritical := CCou = 3;
      end;
  end; { FindCritical}

  procedure GetSignBmp( RS : RhythmStatusType);
  begin
    with TmpBmp.Canvas do
      case RS of
        Positive : CopyRect( ClipRect, BMPRPlus.Canvas, ClipRect);
        Negative : CopyRect( ClipRect, BMPRMinus.Canvas, ClipRect);
        PosToNeg : CopyRect( ClipRect, BMPRDown.Canvas, ClipRect);
        NegToPos : CopyRect( ClipRect, BMPRUp.Canvas, ClipRect);
        end;
  end; { GetSignBmp}

begin
  { Set wait cursor}
  OldCursor := Cursor;
  Screen.Cursor := crHourGlass;

  if SearchForward then TmpSrch := SearchDate + 1
  else begin
    if SearchDate > BirthDate then TmpSrch := SearchDate - 1
    else                           TmpSrch := SearchDate;
    end;

  Find := false;

  { Set Rythms to find}
  RythmSet := [];
  if CBPhys.Checked  then RythmSet := RythmSet + [Physical];
  if CBEmot.Checked  then RythmSet := RythmSet + [Emotional];
  if CBIntel.Checked then RythmSet := RythmSet + [Intelectual];

  { Find Single, Double or Triple}
  if RBSingle.Checked then { Single} CritC := Single
  else
    if RBDouble.Checked then { Double} CritC := Double
    else
      if RBTriple.Checked then { Triple} CritC := Triple;

  while ( not FindCritical) AND ( BirthDate <= TmpSrch) do begin
    if SearchForward then inc( TmpSrch)
    else                  dec( TmpSrch);
    end;

  { Set CriticalDayCal and SearchDate}
  SearchDate := TmpSrch;
  DateToDMY( SearchDate, D, M, Y);
  with CriticalDayCal do begin
    Day := D;
    Month := M;
    Year := Y;
    end;
  { Draw signs}
  for RCou := Physical to Intelectual do begin
    GetRhythmStatus( BirthDate, SearchDate, RCou, RStat);
    GetSignBmp( RStat);
    case RCou of
      Physical    : ImPhysSt.Canvas.Draw( 0, 0, TmpBmp);
      Emotional   : ImEmotSt.Canvas.Draw( 0, 0, TmpBmp);
      Intelectual : ImInteSt.Canvas.Draw( 0, 0, TmpBmp);
      end;
    end;

  { Restore old cursor}
  Screen.Cursor := OldCursor;
end; { TMainF.SearchCriticalDay}

{ Find previous critical day}
procedure TMainF.BSearcPrevClick(Sender: TObject);
begin
  SearchForward := false;
  SearchCriticalDay;
end; { TMainF.Button5Click}

{ Find next critical day}
procedure TMainF.BSearchNextClick(Sender: TObject);
begin
  SearchForward := true;
  SearchCriticalDay;
end; { TMainF.Button4Click}

procedure TMainF.SBVFDayDownClick(Sender: TObject);
var D : integer;
begin
  D := ViewFromCal.Day;
  if D > 1 then dec( D);
  ViewFromCal.Day := D;
end; { TMainF.SpinButton5DownClick}

procedure TMainF.SBVFDayUpClick(Sender: TObject);
var D : integer;
begin
  with ViewFromCal do begin
    D := Day;
    if D < DaysInMonth( Month, Year) then inc( D);
    Day := D;
    end;
end; { TMainF.SpinButton5UpClick}

{ Draw about page}
procedure TMainF.DrawAbout;
begin
{ Image7.Picture.Icon.Handle := LoadIcon( HInstance, 'MAINICON');}
end;

{ Finds nearest day count to fit in bio graph
  Dir - direction ( increasing or decreasing)
  Returns TRUE if Days was changed
}
function NearestDayCou( var Days : integer; Dir : integer) : boolean;
var
  StartPPD, CurPPD : integer;
  AChanged : boolean;

  function Limit : boolean;
  begin
    Limit := ( Days + Dir <= 15) OR ( Days + Dir >= 67);
  end; { Limit}

begin
  AChanged := FALSE;
  with MainF do begin
    StartPPD := BioRGraph.Width DIV Days;
    CurPPD := StartPPD;
    while ( StartPPD = CurPPD) AND NOT Limit do begin
      if NOT AChanged then AChanged := TRUE;
      if NOT Limit then Days := Days + Dir;
      CurPPD := BioRGraph.Width DIV Days;
      end;
    end;
  NearestDayCou := AChanged;
end; { NearestDay}

procedure TMainF.SBMaxDaysDownClick(Sender: TObject);
begin
  if NearestDayCou( DaysInGraph, -1) then begin
    LMaxDay.Caption := Long2Str( DaysInGraph);
    DrawBRGraphic;
    end;
end;

procedure TMainF.SBMaxDaysUpClick(Sender: TObject);
begin
  if NearestDayCou( DaysInGraph, 1) then begin
    LMaxDay.Caption := Long2Str( DaysInGraph);
    DrawBRGraphic;
    end;
end;

end.
